@@ -32,9 +32,9 @@
 # include <mach/clock.h>
 #endif
 
-#include "sudo_compat.h"
-#include "sudo_debug.h"
-#include "sudo_util.h"
+#include <sudo_compat.h>
+#include <sudo_debug.h>
+#include <sudo_util.h>
 
 /*
  * On Linux and FreeBSD, CLOCK_MONOTONIC does not run while sleeping.
