@@ -31,7 +31,7 @@
 #include <string.h>
 #include <errno.h>
 
-#include "sudo_compat.h"
+#include <sudo_compat.h>
 
 #ifndef EAFNOSUPPORT
 # define EAFNOSUPPORT EINVAL
@@ -89,7 +89,7 @@ inet_pton4(const char *src, u_char *dst)
 		const char *pch;
 
 		if ((pch = strchr(digits, ch)) != NULL) {
-			u_int new = *tp * 10 + (pch - digits);
+			unsigned int new = *tp * 10 + (pch - digits);
 
 			if (new > 255)
 				return (0);
@@ -135,7 +135,7 @@ inet_pton6(const char *src, u_char *dst)
 	u_char tmp[NS_IN6ADDRSZ], *tp, *endp, *colonp;
 	const char *xdigits, *curtok;
 	int ch, saw_xdigit, count_xdigit;
-	u_int val;
+	unsigned int val;
 
 	/* cppcheck-suppress uninitvar */
 	memset((tp = tmp), 0, NS_IN6ADDRSZ);
@@ -233,7 +233,7 @@ inet_pton6(const char *src, u_char *dst)
  *	Paul Vixie, 1996.
  */
 int
-sudo_inet_pton(int af, const char *src, void *dst)
+sudo_inet_pton(int af, const char * restrict src, void * restrict dst)
 {
 	switch (af) {
 	case AF_INET:
