@@ -33,15 +33,15 @@
 #ifdef HAVE_STDBOOL_H
 # include <stdbool.h>
 #else
-# include "compat/stdbool.h"
+# include <compat/stdbool.h>
 #endif
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
 
-#include "sudo_compat.h"
-#include "sudo_util.h"
-#include "sudo_debug.h"
+#include <sudo_compat.h>
+#include <sudo_util.h>
+#include <sudo_debug.h>
 
 bool
 sudo_lock_file_v1(int fd, int type)
