@@ -50,7 +50,7 @@ struct uuid {
  * As per RFC 4122 section 4.4.
  */
 void
-sudo_uuid_create_v1(unsigned char uuid_out[16])
+sudo_uuid_create_v1(unsigned char uuid_out[restrict static 16])
 {
     struct uuid uuid;
 
@@ -71,7 +71,7 @@ sudo_uuid_create_v1(unsigned char uuid_out[16])
  * Format a uuid as a 36-byte string (plus one for the NUL).
  */
 char *
-sudo_uuid_to_string_v1(unsigned char uuid[16], char *dst, size_t dstsiz)
+sudo_uuid_to_string_v1(const unsigned char uuid[restrict static 16], char * restrict dst, size_t dstsiz)
 {
     const char hex[] = "0123456789abcdef";
     char *cp = dst;
