@@ -34,8 +34,8 @@
 #include <string.h>
 #include <errno.h>
 
-#include "sudoers.h"
-#include "redblack.h"
+#include <sudoers.h>
+#include <redblack.h>
 #include <gram.h>
 
 /*
@@ -67,7 +67,7 @@ alias_compare(const void *v1, const void *v2)
  */
 struct alias *
 alias_get(const struct sudoers_parse_tree *parse_tree, const char *name,
-    int type)
+    short type)
 {
     struct alias key;
     struct rbnode *node;
@@ -114,8 +114,9 @@ alias_put(struct alias *a)
  * Returns true on success and false on failure, setting errno.
  */
 bool
-alias_add(struct sudoers_parse_tree *parse_tree, char *name, int type,
-    char *file, int line, int column, struct member *members)
+alias_add(struct sudoers_parse_tree *parse_tree, char *name,
+    short type, char *file, int line, int column,
+    struct member *members)
 {
     struct alias *a;
     debug_decl(alias_add, SUDOERS_DEBUG_ALIAS);
@@ -176,22 +177,24 @@ alias_apply_func(void *v1, void *v2)
 /*
  * Apply a function to each alias entry and pass in a cookie.
  */
-void
+bool
 alias_apply(struct sudoers_parse_tree *parse_tree,
     int (*func)(struct sudoers_parse_tree *, struct alias *, void *),
     void *cookie)
 {
     struct alias_apply_closure closure;
+    bool ret = true;
     debug_decl(alias_apply, SUDOERS_DEBUG_ALIAS);
 
     if (parse_tree->aliases != NULL) {
 	closure.parse_tree = parse_tree;
 	closure.func = func;
 	closure.cookie = cookie;
-	rbapply(parse_tree->aliases, alias_apply_func, &closure, inorder);
+	if (rbapply(parse_tree->aliases, alias_apply_func, &closure, inorder) != 0)
+	    ret = false;
     }
 
-    debug_return;
+    debug_return_bool(ret);
 }
 
 /*
@@ -228,7 +231,8 @@ alias_free(void *v)
  * Find the named alias, remove it from the tree and return it.
  */
 struct alias *
-alias_remove(struct sudoers_parse_tree *parse_tree, const char *name, int type)
+alias_remove(struct sudoers_parse_tree *parse_tree, const char *name,
+    short type)
 {
     struct rbnode *node;
     struct alias key;
@@ -262,7 +266,7 @@ free_aliases(struct rbtree *aliases)
 }
 
 const char *
-alias_type_to_string(int alias_type)
+alias_type_to_string(short alias_type)
 {
     return alias_type == HOSTALIAS ? "Host_Alias" :
 	alias_type == CMNDALIAS ? "Cmnd_Alias" :
@@ -277,7 +281,7 @@ alias_type_to_string(int alias_type)
  */
 static bool
 alias_remove_recursive(struct sudoers_parse_tree *parse_tree, char *name,
-    int type, struct rbtree *freelist)
+    short type, struct rbtree *freelist)
 {
     struct member *m;
     struct alias *a;
@@ -299,7 +303,7 @@ alias_remove_recursive(struct sudoers_parse_tree *parse_tree, char *name,
 
 static int
 alias_find_used_members(struct sudoers_parse_tree *parse_tree,
-    struct member_list *members, int atype, struct rbtree *used_aliases)
+    struct member_list *members, short atype, struct rbtree *used_aliases)
 {
     struct member *m;
     int errors = 0;
