@@ -41,7 +41,7 @@
 #include <com_err.h>
 #endif
 
-#include "sudoers.h"
+#include <sudoers.h>
 #include "sudo_auth.h"
 
 #ifdef HAVE_HEIMDAL
@@ -52,7 +52,8 @@
 #endif
 
 #ifndef HAVE_KRB5_VERIFY_USER
-static int verify_krb_v5_tgt(krb5_context, krb5_creds *, char *);
+static int verify_krb_v5_tgt(const struct sudoers_context *, krb5_context,
+    krb5_creds *, const char *);
 #endif
 static struct _sudo_krb5_data {
     krb5_context	sudo_context;
@@ -87,7 +88,8 @@ krb5_get_init_creds_opt_free(krb5_get_init_creds_opt *opts)
 #endif
 
 int
-sudo_krb5_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
+sudo_krb5_setup(const struct sudoers_context *ctx, struct passwd *pw,
+    char **promptp, sudo_auth *auth)
 {
     static char	*krb5_prompt;
     debug_decl(sudo_krb5_init, SUDOERS_DEBUG_AUTH);
@@ -111,16 +113,16 @@ sudo_krb5_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 	 * API does not currently provide this unless the auth is standalone.
 	 */
 	if ((error = krb5_unparse_name(sudo_context, princ, &pname))) {
-	    log_warningx(0,
+	    log_warningx(ctx, 0,
 		N_("%s: unable to convert principal to string ('%s'): %s"),
 		auth->name, pw->pw_name, error_message(error));
 	    debug_return_int(AUTH_FAILURE);
 	}
 
 	if (asprintf(&krb5_prompt, "Password for %s: ", pname) == -1) {
-	    log_warningx(0, N_("unable to allocate memory"));
+	    log_warningx(ctx, 0, N_("unable to allocate memory"));
 	    free(pname);
-	    debug_return_int(AUTH_FATAL);
+	    debug_return_int(AUTH_ERROR);
 	}
 	free(pname);
     }
@@ -130,7 +132,8 @@ sudo_krb5_setup(struct passwd *pw, char **promptp, sudo_auth *auth)
 }
 
 int
-sudo_krb5_init(struct passwd *pw, sudo_auth *auth)
+sudo_krb5_init(const struct sudoers_context *ctx, struct passwd *pw,
+    sudo_auth *auth)
 {
     krb5_context	sudo_context;
     krb5_error_code 	error;
@@ -145,8 +148,8 @@ sudo_krb5_init(struct passwd *pw, sudo_auth *auth)
 	int len = asprintf(&pname, "%s%s%s", pw->pw_name,
 	    sudo_krb5_instance[0] != '/' ? "/" : "", sudo_krb5_instance);
 	if (len == -1) {
-	    log_warningx(0, N_("unable to allocate memory"));
-	    debug_return_int(AUTH_FATAL);
+	    log_warningx(ctx, 0, N_("unable to allocate memory"));
+	    debug_return_int(AUTH_ERROR);
 	}
     }
 
@@ -161,8 +164,8 @@ sudo_krb5_init(struct passwd *pw, sudo_auth *auth)
 
     error = krb5_parse_name(sudo_context, pname, &(sudo_krb5_data.princ));
     if (error) {
-	log_warningx(0, N_("%s: unable to parse '%s': %s"), auth->name, pname,
-	    error_message(error));
+	log_warningx(ctx, 0, N_("%s: unable to parse '%s': %s"), auth->name,
+	    pname, error_message(error));
 	goto done;
     }
 
@@ -170,7 +173,7 @@ sudo_krb5_init(struct passwd *pw, sudo_auth *auth)
 		    (long) getpid());
     if ((error = krb5_cc_resolve(sudo_context, cache_name,
 	&(sudo_krb5_data.ccache)))) {
-	log_warningx(0, N_("%s: unable to resolve credential cache: %s"),
+	log_warningx(ctx, 0, N_("%s: unable to resolve credential cache: %s"),
 	    auth->name, error_message(error));
 	goto done;
     }
@@ -185,7 +188,8 @@ done:
 
 #ifdef HAVE_KRB5_VERIFY_USER
 int
-sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct sudo_conv_callback *callback)
+sudo_krb5_verify(const struct sudoers_context *ctx, struct passwd *pw,
+    const char *pass, sudo_auth *auth, struct sudo_conv_callback *callback)
 {
     krb5_context	sudo_context;
     krb5_principal	princ;
@@ -202,7 +206,8 @@ sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct su
 }
 #else
 int
-sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct sudo_conv_callback *callback)
+sudo_krb5_verify(const struct sudoers_context *ctx, struct passwd *pw,
+    const char *pass, sudo_auth *auth, struct sudo_conv_callback *callback)
 {
     krb5_context	sudo_context;
     krb5_principal	princ;
@@ -219,8 +224,8 @@ sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct su
     /* Set default flags based on the local config file. */
     error = krb5_get_init_creds_opt_alloc(sudo_context, &opts);
     if (error) {
-	log_warningx(0, N_("%s: unable to allocate options: %s"), auth->name,
-	    error_message(error));
+	log_warningx(ctx, 0, N_("%s: unable to allocate options: %s"),
+	    auth->name, error_message(error));
 	goto done;
     }
 #ifdef HAVE_HEIMDAL
@@ -234,7 +239,7 @@ sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct su
 					     NULL, 0, NULL, opts))) {
 	/* Don't print error if just a bad password */
 	if (error != KRB5KRB_AP_ERR_BAD_INTEGRITY) {
-	    log_warningx(0, N_("%s: unable to get credentials: %s"),
+	    log_warningx(ctx, 0, N_("%s: unable to get credentials: %s"),
 		auth->name, error_message(error));
 	}
 	goto done;
@@ -242,15 +247,15 @@ sudo_krb5_verify(struct passwd *pw, const char *pass, sudo_auth *auth, struct su
     creds = &credbuf;
 
     /* Verify the TGT to prevent spoof attacks. */
-    if ((error = verify_krb_v5_tgt(sudo_context, creds, auth->name)))
+    if ((error = verify_krb_v5_tgt(ctx, sudo_context, creds, auth->name)))
 	goto done;
 
     /* Store credential in cache. */
     if ((error = krb5_cc_initialize(sudo_context, ccache, princ))) {
-	log_warningx(0, N_("%s: unable to initialize credential cache: %s"),
+	log_warningx(ctx, 0, N_("%s: unable to initialize credential cache: %s"),
 	    auth->name, error_message(error));
     } else if ((error = krb5_cc_store_cred(sudo_context, ccache, creds))) {
-	log_warningx(0, N_("%s: unable to store credential in cache: %s"),
+	log_warningx(ctx, 0, N_("%s: unable to store credential in cache: %s"),
 	    auth->name, error_message(error));
     }
 
@@ -269,7 +274,8 @@ done:
 #endif
 
 int
-sudo_krb5_cleanup(struct passwd *pw, sudo_auth *auth, bool force)
+sudo_krb5_cleanup(const struct sudoers_context *ctx, struct passwd *pw,
+    sudo_auth *auth, bool force)
 {
     krb5_context	sudo_context;
     krb5_principal	princ;
@@ -301,7 +307,8 @@ sudo_krb5_cleanup(struct passwd *pw, sudo_auth *auth, bool force)
  * Returns 0 for successful authentication, non-zero for failure.
  */
 static int
-verify_krb_v5_tgt(krb5_context sudo_context, krb5_creds *cred, char *auth_name)
+verify_krb_v5_tgt(const struct sudoers_context *ctx, krb5_context sudo_context,
+    krb5_creds *cred, const char *auth_name)
 {
     krb5_error_code	error;
     krb5_principal	server;
@@ -314,8 +321,8 @@ verify_krb_v5_tgt(krb5_context sudo_context, krb5_creds *cred, char *auth_name)
      */
     if ((error = krb5_sname_to_principal(sudo_context, NULL, NULL,
 					KRB5_NT_SRV_HST, &server))) {
-	log_warningx(0, N_("%s: unable to get host principal: %s"), auth_name,
-	    error_message(error));
+	log_warningx(ctx, 0, N_("%s: unable to get host principal: %s"),
+	    auth_name, error_message(error));
 	debug_return_int(-1);
     }
 
@@ -328,7 +335,7 @@ verify_krb_v5_tgt(krb5_context sudo_context, krb5_creds *cred, char *auth_name)
 				   NULL, &vopt);
     krb5_free_principal(sudo_context, server);
     if (error) {
-	log_warningx(0, N_("%s: Cannot verify TGT! Possible attack!: %s"),
+	log_warningx(ctx, 0, N_("%s: Cannot verify TGT! Possible attack!: %s"),
 	    auth_name, error_message(error));
     }
     debug_return_int(error);
