@@ -1,7 +1,7 @@
 /*
  * SPDX-License-Identifier: ISC
  *
- * Copyright (c) 2021-2022 Todd C. Miller <Todd.Miller@sudo.ws>
+ * Copyright (c) 2021-2024 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -44,17 +44,19 @@ new_member(const char *name, short type)
 
     m = calloc(1, sizeof(struct member));
     if (m == NULL)
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	goto oom;
     if (name != NULL) {
 	m->name = strdup(name);
-	if (m->name == NULL) {
-	    sudo_fatalx(U_("%s: %s"), __func__,
-		U_("unable to allocate memory"));
-	}
+	if (m->name == NULL)
+	    goto oom;
     }
     m->type = type;
 
     debug_return_ptr(m);
+oom:
+    sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+    free(m);
+    debug_return_ptr(NULL);
 }
 
 /*
@@ -238,7 +240,7 @@ member_list_equivalent(struct member_list *ml1, struct member_list *ml2)
  * If a host list contains all hosts in bound_hosts, replace them with
  * "ALL".  Also prune hosts on either side of "ALL" when possible.
  */
-static void
+static bool
 simplify_host_list(struct member_list *hosts, const char *file, int line,
     int column, struct member_list *bound_hosts)
 {
@@ -290,6 +292,8 @@ simplify_host_list(struct member_list *hosts, const char *file, int line,
 		}
 	    }
 	    m = new_member(NULL, ALL);
+	    if (m == NULL)
+		debug_return_bool(false);
 	    TAILQ_INSERT_TAIL(hosts, m, entries);
 	}
     }
@@ -316,7 +320,7 @@ simplify_host_list(struct member_list *hosts, const char *file, int line,
 	}
     }
 
-    debug_return;
+    debug_return_bool(true);
 }
 
 /*
@@ -349,8 +353,10 @@ alias_make_unique(const char *old_name, short type,
     for (;;) {
 	suffix++;
 	free(new_name);
-	if (asprintf(&new_name, "%.*s_%lld", (int)namelen, old_name, suffix) == -1)
-	    sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	if (asprintf(&new_name, "%.*s_%lld", (int)namelen, old_name, suffix) == -1) {
+	    sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	    debug_return_ptr(NULL);
+	}
 	/* Make sure new_name is not already in use. */
 	a = alias_get(merged_tree, new_name, type);
 	if (a != NULL) {
@@ -395,8 +401,10 @@ alias_rename_members(struct sudoers_parse_tree *parse_tree, struct alias *a,
     TAILQ_FOREACH(m, &a->members, entries) {
 	if (m->type == ALIAS && strcmp(m->name, closure->old_name) == 0) {
 	    char *copy = strdup(closure->new_name);
-	    if (copy == NULL)
-		sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	    if (copy == NULL) {
+		sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+		debug_return_int(-1);
+	    }
 	    free(m->name);
 	    m->name = copy;
 	}
@@ -405,7 +413,7 @@ alias_rename_members(struct sudoers_parse_tree *parse_tree, struct alias *a,
     debug_return_int(0);
 }
 
-static void
+static bool
 alias_rename_defaults(const char *old_name, const char *new_name,
     short alias_type, struct defaults_list *defaults)
 {
@@ -442,8 +450,9 @@ alias_rename_defaults(const char *old_name, const char *new_name,
 	    if (strcmp(m->name, old_name) == 0) {
 		char *copy = strdup(new_name);
 		if (copy == NULL) {
-		    sudo_fatalx(U_("%s: %s"), __func__,
+		    sudo_warnx(U_("%s: %s"), __func__,
 			U_("unable to allocate memory"));
+		    debug_return_bool(false);
 		}
 		free(m->name);
 		m->name = copy;
@@ -451,10 +460,10 @@ alias_rename_defaults(const char *old_name, const char *new_name,
 	}
     }
 
-    debug_return;
+    debug_return_bool(true);
 }
 
-static void
+static bool
 alias_rename_member(const char *old_name, const char *new_name,
     struct member *m)
 {
@@ -463,17 +472,18 @@ alias_rename_member(const char *old_name, const char *new_name,
     if (m->type == ALIAS && strcmp(m->name, old_name) == 0) {
 	char *copy = strdup(new_name);
 	if (copy == NULL) {
-	    sudo_fatalx(U_("%s: %s"), __func__,
+	    sudo_warnx(U_("%s: %s"), __func__,
 		U_("unable to allocate memory"));
+	    debug_return_bool(false);
 	}
 	free(m->name);
 	m->name = copy;
     }
 
-    debug_return;
+    debug_return_bool(true);
 }
 
-static void
+static bool
 alias_rename_member_list(const char *old_name, const char *new_name,
     struct member_list *members)
 {
@@ -481,10 +491,11 @@ alias_rename_member_list(const char *old_name, const char *new_name,
     debug_decl(alias_rename_member_list, SUDOERS_DEBUG_ALIAS);
 
     TAILQ_FOREACH(m, members, entries) {
-	alias_rename_member(old_name, new_name, m);
+	if (!alias_rename_member(old_name, new_name, m))
+	    debug_return_bool(false);
     }
 
-    debug_return;
+    debug_return_bool(true);
 }
 
 static bool
@@ -494,45 +505,57 @@ alias_rename_userspecs(const char *old_name, const char *new_name,
     struct privilege *priv;
     struct cmndspec *cs;
     struct userspec *us;
-    bool ret = true;
     debug_decl(alias_rename_userspecs, SUDOERS_DEBUG_ALIAS);
 
     TAILQ_FOREACH(us, userspecs, entries) {
 	if (alias_type == USERALIAS) {
-	    alias_rename_member_list(old_name, new_name, &us->users);
+	    if (!alias_rename_member_list(old_name, new_name, &us->users)) {
+		debug_return_bool(false);
+	    }
 	}
 	TAILQ_FOREACH(priv, &us->privileges, entries) {
-	    alias_rename_defaults(old_name, new_name, alias_type, &priv->defaults);
+	    if (!alias_rename_defaults(old_name, new_name, alias_type, &priv->defaults)) {
+		debug_return_bool(false);
+	    }
 	    if (alias_type == HOSTALIAS) {
-		alias_rename_member_list(old_name, new_name, &priv->hostlist);
+		if (!alias_rename_member_list(old_name, new_name, &priv->hostlist)) {
+		    debug_return_bool(false);
+		}
 		continue;
 	    }
 	    TAILQ_FOREACH(cs, &priv->cmndlist, entries) {
 		if (alias_type == CMNDALIAS) {
-		    alias_rename_member(old_name, new_name, cs->cmnd);
+		    if (!alias_rename_member(old_name, new_name, cs->cmnd)) {
+			debug_return_bool(false);
+		    }
 		    continue;
 		}
 		if (alias_type == RUNASALIAS) {
 		    if (cs->runasuserlist != NULL) {
-			alias_rename_member_list(old_name, new_name, cs->runasuserlist);
+			if (!alias_rename_member_list(old_name, new_name, cs->runasuserlist)) {
+			    debug_return_bool(false);
+			}
 		    }
 		    if (cs->runasgrouplist != NULL) {
-			alias_rename_member_list(old_name, new_name, cs->runasgrouplist);
+			if (!alias_rename_member_list(old_name, new_name, cs->runasgrouplist)) {
+			    debug_return_bool(false);
+			}
 		    }
 		}
 	    }
 	}
     }
 
-    debug_return_bool(ret);
+    debug_return_bool(true);
 }
 
 /*
  * Rename an alias in parse_tree and all the places where it is used.
+ * Takes ownership if new_name which must not be freed by the caller.
  */
 static bool
-alias_rename(const char *old_name, const char *new_name,
-    short alias_type, struct sudoers_parse_tree *parse_tree)
+alias_rename(const char *old_name, char *new_name, short alias_type,
+    struct sudoers_parse_tree *parse_tree)
 {
     struct alias_rename_closure closure = { old_name, new_name, alias_type };
     struct alias *a;
@@ -543,14 +566,13 @@ alias_rename(const char *old_name, const char *new_name,
     if (a == NULL) {
 	/* Should not happen. */
 	sudo_warnx(U_("unable to find alias %s"), old_name);
+	free(new_name);
 	debug_return_bool(false);
     }
     log_warnx(U_("%s:%d:%d: renaming alias %s to %s"),
 	a->file, a->line, a->column, a->name, new_name);
     free(a->name);
-    a->name = strdup(new_name);
-    if (a->name == NULL)
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+    a->name = new_name;
     switch (rbinsert(parse_tree->aliases, a, NULL)) {
     case 0:
 	/* success */
@@ -559,19 +581,26 @@ alias_rename(const char *old_name, const char *new_name,
 	/* Already present, should not happen. */
 	errno = EEXIST;
 	sudo_warn(U_("%s: %s"), __func__, a->name);
+	alias_free(a);
+	debug_return_bool(false);
 	break;
     default:
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	alias_free(a);
+	debug_return_bool(false);
     }
 
     /* Rename it in the aliases tree itself (aliases can be nested). */
-    alias_apply(parse_tree, alias_rename_members, &closure);
+    if (!alias_apply(parse_tree, alias_rename_members, &closure))
+	debug_return_bool(false);
 
     /* Rename it in the Defaults list. */
-    alias_rename_defaults(old_name, new_name, alias_type, &parse_tree->defaults);
+    if (!alias_rename_defaults(old_name, new_name, alias_type, &parse_tree->defaults))
+	debug_return_bool(false);
 
     /* Rename it in the userspecs list. */
-    alias_rename_userspecs(old_name, new_name, alias_type, &parse_tree->userspecs);
+    if (!alias_rename_userspecs(old_name, new_name, alias_type, &parse_tree->userspecs))
+	debug_return_bool(false);
 
     debug_return_bool(true);
 }
@@ -610,8 +639,10 @@ alias_resolve_conflicts(struct sudoers_parse_tree *parse_tree0, struct alias *a,
 
 	/* Rename alias 'b' to avoid a naming conflict. */
 	new_name = alias_make_unique(a->name, a->type, parse_tree, merged_tree);
-	alias_rename(a->name, new_name, a->type, parse_tree);
-	free(new_name);
+	if (new_name == NULL)
+	    debug_return_int(-1);
+	if (!alias_rename(a->name, new_name, a->type, parse_tree))
+	    debug_return_int(-1);
     }
 
     /*
@@ -628,9 +659,10 @@ alias_resolve_conflicts(struct sudoers_parse_tree *parse_tree0, struct alias *a,
 	/* already present, should not happen. */
 	errno = EEXIST;
 	sudo_warn(U_("%s: %s"), __func__, a->name);
-	break;
+	debug_return_int(-1);
     default:
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	debug_return_int(-1);
     }
 
     debug_return_int(0);
@@ -657,7 +689,8 @@ merge_aliases(struct sudoers_parse_tree_list *parse_trees,
 	 * Resolve any conflicts in alias names, renaming aliases as
 	 * needed and eliminating duplicates.
 	 */
-	alias_apply(parse_tree, alias_resolve_conflicts, merged_tree);
+	if (!alias_apply(parse_tree, alias_resolve_conflicts, merged_tree))
+	    debug_return_bool(false);
 
 	/*
 	 * Destroy the old alias tree without freeing the alias data
@@ -766,7 +799,8 @@ defaults_list_equivalent(struct defaults_list *dl1, struct defaults_list *dl2)
 enum cvtsudoers_conflict {
     CONFLICT_NONE,
     CONFLICT_RESOLVED,
-    CONFLICT_UNRESOLVED
+    CONFLICT_UNRESOLVED,
+    CONFLICT_ERROR
 };
 
 /*
@@ -806,11 +840,15 @@ defaults_check_conflict(struct defaults *def,
 			 */
 			if (d->type == DEFAULTS) {
 			    struct member *m = new_member(NULL, ALL);
+			    if (m == NULL)
+				debug_return_int(CONFLICT_ERROR);
 			    TAILQ_INSERT_TAIL(&d->binding->members, m, entries);
 			    d->type = def->type;
 			}
 			if (def->type == DEFAULTS) {
 			    struct member *m = new_member(NULL, ALL);
+			    if (m == NULL)
+				debug_return_int(CONFLICT_ERROR);
 			    TAILQ_INSERT_TAIL(&def->binding->members, m, entries);
 			    def->type = d->type;
 			}
@@ -861,6 +899,8 @@ merge_defaults(struct sudoers_parse_tree_list *parse_trees,
 	TAILQ_FOREACH(def, &parse_tree->defaults, entries) {
 	    if (parse_tree->lhost != NULL && def->type == DEFAULTS) {
 		m = new_member(parse_tree->lhost, WORD);
+		if (m == NULL)
+		    debug_return_bool(false);
 		log_warnx(U_("%s:%d:%d: made Defaults \"%s\" specific to host %s"),
 		    def->file, def->line, def->column, def->var,
 		    parse_tree->lhost);
@@ -893,6 +933,10 @@ merge_defaults(struct sudoers_parse_tree_list *parse_trees,
 		    def->file, def->line, def->column, def->var);
 		free_default(def);
 		break;
+	    default:
+		/* warning printed by defaults_check_conflict() */
+		free_default(def);
+		debug_return_bool(false);
 	    }
 	}
     }
@@ -903,8 +947,10 @@ merge_defaults(struct sudoers_parse_tree_list *parse_trees,
     TAILQ_FOREACH(def, &merged_tree->defaults, entries) {
 	/* TODO: handle refcnt != 1 */
 	if (def->type == DEFAULTS_HOST && def->binding->refcnt == 1) {
-	    simplify_host_list(&def->binding->members, def->file, def->line,
-		def->column, bound_hosts);
+	    if (!simplify_host_list(&def->binding->members, def->file,
+		    def->line, def->column, bound_hosts)) {
+		debug_return_bool(false);
+	    }
 	    m = TAILQ_FIRST(&def->binding->members);
 	    if (m->type == ALL && !m->negated) {
 		if (TAILQ_NEXT(m, entries) == NULL) {
@@ -962,7 +1008,6 @@ cmndspec_equivalent(struct cmndspec *cs1, struct cmndspec *cs2, bool check_negat
     } else if (cs1->runchroot != cs2->runchroot) {
 	debug_return_bool(false);
     }
-#ifdef HAVE_SELINUX
     if (cs1->role != NULL && cs2->role != NULL) {
 	if (strcmp(cs1->role, cs2->role) != 0)
 	    debug_return_bool(false);
@@ -975,16 +1020,12 @@ cmndspec_equivalent(struct cmndspec *cs1, struct cmndspec *cs2, bool check_negat
     } else if (cs1->type != cs2->type) {
 	debug_return_bool(false);
     }
-#endif
-#ifdef HAVE_APPARMOR
     if (cs1->apparmor_profile != NULL && cs2->apparmor_profile != NULL) {
 	if (strcmp(cs1->apparmor_profile, cs2->apparmor_profile) != 0)
 	    debug_return_bool(false);
     } else if (cs1->apparmor_profile != cs2->apparmor_profile) {
 	debug_return_bool(false);
     }
-#endif
-#ifdef HAVE_PRIV_SET
     if (cs1->privs != NULL && cs2->privs != NULL) {
 	if (strcmp(cs1->privs, cs2->privs) != 0)
 	    debug_return_bool(false);
@@ -997,7 +1038,6 @@ cmndspec_equivalent(struct cmndspec *cs1, struct cmndspec *cs2, bool check_negat
     } else if (cs1->limitprivs != cs2->limitprivs) {
 	debug_return_bool(false);
     }
-#endif
 
     debug_return_bool(true);
 }
@@ -1151,12 +1191,14 @@ merge_userspecs(struct sudoers_parse_tree_list *parse_trees,
 		TAILQ_FOREACH(m, &priv->hostlist, entries) {
 		    /* We don't alter !ALL in a hostlist (XXX - should we?). */
 		    if (m->type == ALL && !m->negated) {
-			m->type = WORD;
-			m->name = strdup(parse_tree->lhost);
-			if (m->name == NULL) {
-			    sudo_fatalx(U_("%s: %s"), __func__,
+			char *copy = strdup(parse_tree->lhost);
+			if (copy == NULL) {
+			    sudo_warnx(U_("%s: %s"), __func__,
 				U_("unable to allocate memory"));
+			    debug_return_bool(false);
 			}
+			m->type = WORD;
+			m->name = copy;
 		    }
 		}
 	    }
@@ -1183,6 +1225,10 @@ merge_userspecs(struct sudoers_parse_tree_list *parse_trees,
 		    us->file, us->line, us->column);
 		free_userspec(us);
 		break;
+	    default:
+		/* warning printed by defaults_check_conflict() */
+		free_userspec(us);
+		debug_return_bool(false);
 	    }
 	}
     }
@@ -1195,8 +1241,10 @@ merge_userspecs(struct sudoers_parse_tree_list *parse_trees,
     TAILQ_FOREACH_REVERSE(us, &merged_tree->userspecs, userspec_list, entries) {
 	TAILQ_FOREACH_REVERSE(priv, &us->privileges, privilege_list, entries) {
 	    /* TODO: simplify other lists? */
-	    simplify_host_list(&priv->hostlist, us->file, us->line, us->column,
-		bound_hosts);
+	    if (!simplify_host_list(&priv->hostlist, us->file, us->line,
+		    us->column, bound_hosts)) {
+		debug_return_bool(false);
+	    }
 	}
     }
 
@@ -1223,12 +1271,16 @@ merge_sudoers(struct sudoers_parse_tree_list *parse_trees,
     if (parse_tree == NULL) {
 	TAILQ_FOREACH(parse_tree, parse_trees, entries) {
 	    struct member *m = new_member(parse_tree->lhost, WORD);
+	    if (m == NULL)
+		goto bad;
 	    TAILQ_INSERT_TAIL(&bound_hosts, m, entries);
 	}
     }
 
-    if ((merged_tree->aliases = alloc_aliases()) == NULL)
-	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+    if ((merged_tree->aliases = alloc_aliases()) == NULL) {
+	sudo_warnx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
+	goto bad;
+    }
 
     if (!merge_aliases(parse_trees, merged_tree))
 	goto bad;
