@@ -130,10 +130,12 @@ dump_defaults(void)
 		    sudo_printf(SUDO_CONV_INFO_MSG, "\n");
 		    break;
 		case T_TIMESPEC: {
-		    /* display timespec in minutes as a double */
-		    double d = (double)cur->sd_un.tspec.tv_sec +
-			((double)cur->sd_un.tspec.tv_nsec / 1000000000.0);
-		    sudo_printf(SUDO_CONV_INFO_MSG, desc, d / 60.0);
+		    /* display timespec in minutes and 10ths of a minute */
+		    const int min = cur->sd_un.tspec.tv_sec / 60;
+		    int decimin =
+			(((cur->sd_un.tspec.tv_sec % 60) * 10) + 30) / 60;
+		    decimin += cur->sd_un.tspec.tv_nsec / 100000000;
+		    sudo_printf(SUDO_CONV_INFO_MSG, desc, min, decimin);
 		    sudo_printf(SUDO_CONV_INFO_MSG, "\n");
 		    break;
 		}
@@ -481,7 +483,7 @@ init_defaults(void)
 #ifdef LONG_OTP_PROMPT
     def_long_otp_prompt = true;
 #endif
-#ifdef IGNORE_DOT_PATH
+#ifndef ALLOW_DOT_PATH
     def_ignore_dot = true;
 #endif
 #ifdef ALWAYS_SEND_MAIL
