@@ -24,17 +24,17 @@
 #include <errno.h>
 #include <limits.h>
 
-#include "sudo_compat.h"
-#include "sudo_dso.h"
-#include "sudo_util.h"
-#include "sudo_fatal.h"
+#include <sudo_compat.h>
+#include <sudo_dso.h>
+#include <sudo_util.h>
+#include <sudo_fatal.h>
 
 sudo_dso_public int main(int argc, char *argv[]);
 
-static void
+sudo_noreturn static void
 usage(void)
 {
-    fprintf(stderr, "usage: %s plugin.so symbols_file\n", getprogname());
+    fprintf(stderr, "usage: %s [-v] plugin.so symbols_file\n", getprogname());
     exit(EXIT_FAILURE);
 }
 
@@ -46,14 +46,27 @@ main(int argc, char *argv[])
     const char *symbols_file;
     char *cp, line[LINE_MAX];
     FILE *fp;
-    int ntests = 0, errors = 0;
+    int ch, ntests = 0, errors = 0;
 
     initprogname(argc > 0 ? argv[0] : "check_symbols");
 
-    if (argc != 3)
+    while ((ch = getopt(argc, argv, "v")) != -1) {
+	switch (ch) {
+	case 'v':
+	    /* ignored */
+	    break;
+	default:
+	    usage();
+	    /* NOTREACHED */
+	}
+    }
+    argc -= optind;
+    argv += optind;
+
+    if (argc != 2)
 	usage();
-    plugin_path = argv[1];
-    symbols_file = argv[2];
+    plugin_path = argv[0];
+    symbols_file = argv[1];
 
     handle = sudo_dso_load(plugin_path, SUDO_DSO_LAZY|SUDO_DSO_GLOBAL);
     if (handle == NULL) {
@@ -92,8 +105,10 @@ main(int argc, char *argv[])
 
     sudo_dso_unload(handle);
 
-    printf("%s: %d tests run, %d errors, %d%% success rate\n", getprogname(),
-	ntests, errors, (ntests - errors) * 100 / ntests);
+    if (ntests != 0) {
+	printf("%s: %d tests run, %d errors, %d%% success rate\n",
+	    getprogname(), ntests, errors, (ntests - errors) * 100 / ntests);
+    }
 
     exit(errors);
 }
