@@ -3,7 +3,9 @@
 # Test cvtsudoers.conf with padding
 #
 
-./cvtsudoers -c $TESTDIR/test31.conf <<EOF
+: ${CVTSUDOERS=cvtsudoers}
+
+$CVTSUDOERS -c $TESTDIR/test31.conf <<EOF
 Defaults authenticate, timestamp_timeout=0
 User_Alias FULLTIMERS = user1, user2, user3
 
