@@ -3,7 +3,9 @@
 # Test cvtsudoers.conf with invalid padding
 #
 
-./cvtsudoers -c "" -b "ou=SUDOers,dc=my-domain,dc=com" -O 1000 -P 1 <<EOF
+: ${CVTSUDOERS=cvtsudoers}
+
+$CVTSUDOERS -c "" -b "ou=SUDOers,dc=my-domain,dc=com" -O 1000 -P 1 <<EOF
 user0  ALL = (ALL:ALL) ALL
 user1  ALL = (ALL:ALL) ALL
 user2  ALL = (ALL:ALL) ALL
