@@ -27,12 +27,10 @@
 
 #define SUDO_ERROR_WRAP 0
 
-#include "sudoers.h"
+#include <sudoers.h>
 
 #include <def_data.c>
 
-struct sudo_user sudo_user;
-
 struct test_data {
     const char *input;
     const char *output;
@@ -53,7 +51,7 @@ sudo_dso_public int main(int argc, char *argv[]);
 int
 main(int argc, char *argv[])
 {
-    int ntests = 0, errors = 0;
+    int ch, ntests = 0, errors = 0;
     struct test_data *td;
     struct passwd *pw;
     char *path = NULL;
@@ -61,6 +59,19 @@ main(int argc, char *argv[])
 
     initprogname(argc > 0 ? argv[0] : "check_exptilde");
 
+    while ((ch = getopt(argc, argv, "v")) != -1) {
+	switch (ch) {
+	case 'v':
+	    /* ignored */
+	    break;
+	default:
+	    fprintf(stderr, "usage: %s [-v]\n", getprogname());
+	    return EXIT_FAILURE;
+	}
+    }
+    argc -= optind;
+    argv += optind;
+
     /* Prime the passwd cache */
     pw = sudo_mkpwent("root", 0, 0, "/", "/bin/sh");
     if (pw == NULL)
@@ -74,7 +85,6 @@ main(int argc, char *argv[])
 
     for (td = test_data; td->input != NULL; td++) {
 	ntests++;
-	free(path);
 	if ((path = strdup(td->input)) == NULL)
 	    sudo_fatal(NULL);
 	result = expand_tilde(&path, td->user);
@@ -86,14 +96,12 @@ main(int argc, char *argv[])
 	    } else {
 		sudo_warnx("unexpected failure: input %s", td->input);
 	    }
-	    continue;
-	}
-	if (td->result && strcmp(path, td->output) != 0) {
+	} else if (td->result && strcmp(path, td->output) != 0) {
 	    errors++;
 	    sudo_warnx("incorrect output for input %s: expected %s, got %s",
 		td->input, td->output, path);
-	    continue;
 	}
+	free(path);
     }
 
     if (ntests != 0) {
@@ -101,5 +109,5 @@ main(int argc, char *argv[])
 	    getprogname(), ntests, errors, (ntests - errors) * 100 / ntests);
     }
 
-    exit(errors);
+    return errors;
 }
