@@ -27,13 +27,13 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#include "sudoers.h"
+#include <sudoers.h>
 
 /*
  * Like strlcpy() but collapses non-space chars escaped with a backslash.
  */
 size_t
-strlcpy_unescape(char *dst, const char *src, size_t size)
+strlcpy_unescape(char *restrict dst, const char *restrict src, size_t size)
 {
     size_t len = 0;
     char ch;
