@@ -49,7 +49,6 @@
 #include <defaults.h>
 #include <logging.h>
 #include <parse.h>
-#include <pivot.h>
 
 /*
  * Info passed in from the sudo front-end.
@@ -314,15 +313,16 @@ struct stat;
  * Function prototypes
  */
 /* goodpath.c */
-bool sudo_goodpath(const char *path, struct stat *sbp);
+bool sudo_goodpath(const char *path, const char *runchroot, struct stat *sbp);
 
 /* findpath.c */
 int find_path(const char *infile, char **outfile, struct stat *sbp,
-    const char *path, bool ignore_dot, char * const *allowlist);
+    const char *path, const char *runchroot, bool ignore_dot,
+    char * const *allowlist);
 
 /* resolve_cmnd.c */
 int resolve_cmnd(struct sudoers_context *ctx, const char *infile,
-    char **outfile, const char *path);
+    char **outfile, const char *path, const char *runchroot);
 
 /* check.c */
 int check_user(struct sudoers_context *ctx, unsigned int validated, unsigned int mode);
@@ -496,10 +496,10 @@ void canon_path_free(char *resolved);
 void canon_path_free_cache(void);
 
 /* strlcpy_unesc.c */
-size_t strlcpy_unescape(char *restrict dst, const char *restrict src, size_t size);
+size_t strlcpy_unescape(char * restrict dst, const char * restrict src, size_t size);
 
 /* strvec_join.c */
-char *strvec_join(char *const argv[], char sep, size_t (*cpy)(char *, const char *, size_t));
+char *strvec_join(char *const argv[], char sep, size_t (*cpy)(char * restrict, const char * restrict, size_t));
 
 /* unesc_str.c */
 void unescape_string(char *str);
