@@ -33,7 +33,7 @@
 #include <string.h>
 #include <errno.h>
 
-#include "sudoers.h"
+#include <sudoers.h>
 
 /*
  * Similar to setenv(3) but operates on a private copy of the environment.
@@ -101,7 +101,7 @@ sudoers_hook_getenv(const char *name, char **value, void *closure)
 	    goto done;
 	}
 	if (strcmp(name, "LC_ALL") == 0 || strcmp(name, "LC_MESSAGES") == 0) {
-	    *value = (char *)def_sudoers_locale;
+	    *value = def_sudoers_locale;
 	    goto done;
 	}
     }
