@@ -31,10 +31,10 @@
 #include <fcntl.h>
 #include <signal.h>
 
-#include "sudo.h"
-#include "sudo_exec.h"
-#include "sudo_plugin.h"
-#include "sudo_plugin_int.h"
+#include <sudo.h>
+#include <sudo_exec.h>
+#include <sudo_plugin.h>
+#include <sudo_plugin_int.h>
 
 int io_fds[6] = { -1, -1, -1, -1, -1, -1 };
 
