@@ -279,8 +279,7 @@ mon_errsock_cb(int fd, int what, void *v)
      * Note that the error socket is *blocking*.
      */
     nread = read(fd, &errval, sizeof(errval));
-    switch (nread) {
-    case -1:
+    if (nread < 0) {
 	if (errno != EAGAIN && errno != EINTR) {
 	    if (mc->cstat->val == CMD_INVALID) {
 		/* XXX - need a way to distinguish non-exec error. */
@@ -291,22 +290,22 @@ mon_errsock_cb(int fd, int what, void *v)
 		"%s: failed to read error socket", __func__);
 	    sudo_ev_loopbreak(mc->evbase);
 	}
-	break;
-    default:
-	if (nread == 0) {
-	    /* The error socket closes when the command is executed. */
-	    sudo_debug_printf(SUDO_DEBUG_INFO, "EOF on error socket");
-	} else {
-	    /* Errno value when child is unable to execute command. */
-	    sudo_debug_printf(SUDO_DEBUG_INFO, "errno from child: %s",
-		strerror(errval));
-	    mc->cstat->type = CMD_ERRNO;
-	    mc->cstat->val = errval;
-	}
-	sudo_ev_del(mc->evbase, mc->errsock_event);
-	close(fd);
-	break;
+	debug_return;
     }
+
+    if (nread == 0) {
+	/* The error socket closes when the command is executed. */
+	sudo_debug_printf(SUDO_DEBUG_INFO, "EOF on error socket");
+    } else {
+	/* Errno value when child is unable to execute command. */
+	sudo_debug_printf(SUDO_DEBUG_INFO, "errno from child: %s",
+	    strerror(errval));
+	mc->cstat->type = CMD_ERRNO;
+	mc->cstat->val = errval;
+    }
+    sudo_ev_del(mc->evbase, mc->errsock_event);
+    close(fd);
+
     debug_return;
 }
 
@@ -408,22 +407,34 @@ exec_cmnd_pty(struct command_details *details, sigset_t *mask,
 }
 
 /*
- * Fill in the monitor closure and setup initial events.
- * Allocates read events for the signal pipe, error pipe and backchannel.
+ * Fill in the non-event part of the monitor closure.
  */
 static void
-fill_exec_closure_monitor(struct monitor_closure *mc,
+init_exec_closure_monitor(struct monitor_closure *mc,
     const struct command_details *details, struct command_status *cstat,
-    int errfd, int backchannel)
+    int backchannel)
 {
-    debug_decl(fill_exec_closure_monitor, SUDO_DEBUG_EXEC);
+    debug_decl(init_exec_closure_monitor, SUDO_DEBUG_EXEC);
     
     /* Fill in the non-event part of the closure. */
+    memset(mc, 0, sizeof(*mc));
     mc->details = details;
     mc->cstat = cstat;
     mc->backchannel = backchannel;
     mc->mon_pgrp = getpgrp();
 
+    debug_return;
+}
+
+/*
+ * Fill in the monitor closure and setup initial events.
+ * Allocates read events for the signal pipe, error pipe and backchannel.
+ */
+static void
+init_exec_events_monitor(struct monitor_closure *mc, int errfd)
+{
+    debug_decl(init_exec_events_monitor, SUDO_DEBUG_EXEC);
+
     /* Setup event base and events. */
     mc->evbase = sudo_ev_base_alloc();
     if (mc->evbase == NULL)
@@ -438,7 +449,7 @@ fill_exec_closure_monitor(struct monitor_closure *mc,
 	sudo_fatal("%s", U_("unable to add event to queue"));
 
     /* Event for forwarded signals via backchannel. */
-    mc->backchannel_event = sudo_ev_alloc(backchannel,
+    mc->backchannel_event = sudo_ev_alloc(mc->backchannel,
 	SUDO_EV_READ|SUDO_EV_PERSIST, mon_backchannel_cb, mc);
     if (mc->backchannel_event == NULL)
 	sudo_fatalx(U_("%s: %s"), __func__, U_("unable to allocate memory"));
@@ -543,7 +554,7 @@ int
 exec_monitor(struct command_details *details, sigset_t *oset,
     bool foreground, int backchannel, int intercept_fd)
 {
-    struct monitor_closure mc = { 0 };
+    struct monitor_closure mc;
     struct command_status cstat;
     struct sigaction sa;
     int errsock[2];
@@ -579,6 +590,9 @@ exec_monitor(struct command_details *details, sigset_t *oset,
 	goto bad;
     }
 
+    /* Fill in exec closure after creating a new session. */
+    init_exec_closure_monitor(&mc, details, &cstat, backchannel);
+
     /*
      * The child waits on the other end of a socketpair for the
      * parent to set the controlling terminal.  It also writes
@@ -649,7 +663,7 @@ exec_monitor(struct command_details *details, sigset_t *oset,
      * Create new event base and register read events for the
      * signal pipe, error pipe, and backchannel.
      */
-    fill_exec_closure_monitor(&mc, details, &cstat, errsock[0], backchannel);
+    init_exec_events_monitor(&mc, errsock[0]);
 
     /* Restore signal mask now that signal handlers are setup. */
     sigprocmask(SIG_SETMASK, oset, NULL);
