@@ -105,7 +105,6 @@
 #define ARG_ASKPASS		26
 #define ARG_INTERCEPT_SETID	27
 #define ARG_INTERCEPT_PTRACE	28
-#define ARG_APPARMOR_PROFILE	29
 
 /*
  * Flags for tgetpass()
@@ -317,8 +316,7 @@ char *getenv_unhooked(const char *name);
 int get_net_ifs(char **addrinfo);
 
 /* ttyname.c */
-char *get_process_ttyname(char *name, size_t namelen);
-bool sudo_isatty(int fd, struct stat *sb);
+dev_t get_process_ttyname(char *name, size_t namelen);
 
 /* signal.c */
 struct sigaction;
